import {
  RestApiRequest,
  RestApiRoute,
  RestPathParams,
  RestQueryParams,
  RestRequestBody,
  RestRequestHeaders,
  RestRequestHeadersWithAuthorization,
  RestRequireAuthentication,
  RestResponseBody,
} from '../types';
import {OmitNever} from '../types/internal';


export async function fetchRestApi<
    PathParams extends RestPathParams,
    QueryParams extends RestQueryParams,
    RequestHeaders extends RestRequestHeaders,
    RequestBody extends RestRequestBody,
    ResponseBody extends RestResponseBody,
    RequireAuthentication extends RestRequireAuthentication>(
    restApiUrl: string,
    route: RestApiRoute<PathParams, QueryParams, RequestHeaders, RequestBody, ResponseBody, RequireAuthentication>,
    request: OmitNever<RestApiRequest<PathParams, QueryParams, RestRequestHeadersWithAuthorization<RequestHeaders, RequireAuthentication>, RequestBody>>,
): Promise<ResponseBody> {
  const pathParams = 'pathParams' in request ? request.pathParams as PathParams : undefined as never;
  const queryParams = 'queryParams' in request ? request.queryParams as QueryParams : undefined as never;
  const headers = 'headers' in request ? request.headers as RequestHeaders : undefined as never;
  const body = 'body' in request ? request.body as RequestBody : undefined as never;

  const url = `${restApiUrl}${route.buildUrl(pathParams, queryParams)}`;
  const response = await fetch(
      url,
      {
        method: route.method,
        headers: headers,
        body: body ? JSON.stringify(body) : null,
      });

  if (!response.ok && response.status !== 404)
    throw `Request ${route.method} ${url} failed`;

  const text = (await response.text()).trim();
  return response.ok && response.status !== 204
      ? text ? JSON.parse(text) : null
      : null;
}
